# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import re
from django.conf import settings
from django.core.urlresolvers import RegexURLResolver
import sys


def url_pattern(url_pattern_regex, url_parent=''):
    callback = None
    url = url_parent
    if url and url[0] == '^':
        url = url[1:]

    url_parent = url_pattern_regex._regex

    try:
        callback = url_pattern_regex.callback.__dict__
        if 'cls' in callback and all(list(filter(lambda x: x.__name__ == 'APIView', callback['cls'].mro()))):
            callback = callback['cls']
        else:
            callback = None
    except AttributeError:
        pass

    if url_parent and url_parent[0] == '^':
        url_parent = url_parent[1:]

    url = '/%s%s' % (url, url_parent)
    url = url.replace('\.(?P<format>\w+)$', '.api').replace('?P', '')
    is_sep = url[-2:] == '/$' or url[-3:] == '/?$'
    format_name = url.split('.').pop() if url.split('.').pop() == 'api' else ''
    if format_name == 'api':
        url = '.'.join(url.split('.')[:-1])
    url = re.sub(r'[\?\$\^]+', '', url, re.IGNORECASE)
    url = url.strip('/').split('/')

    def ulr_format(item):
        arg = ':arg'
        item = re.sub(r'(\\\w?)+', '', item, re.IGNORECASE)
        p = re.compile(ur'\<(?P<name>\w+)\>', re.IGNORECASE)
        match = re.findall(p, item)

        if len(match) == 1:
            arg = match[0]

        item = re.sub(r'([^a-zА-Я0-9_-])+', '', item, re.IGNORECASE)

        if item == '':
            item = arg
        elif arg == item:
            item = ':%s' % arg
        return item

    url = list(map(ulr_format, url))

    if not all(url) or len(url) == 1 and url[0] == ':arg':
        url = ['']
    else:
        url = [''] + url

    url = '/'.join(url)
    if format_name == 'api':
        url = '%s.api' % url
    elif is_sep:
        url = '%s/' % url

    return url, callback


def search_ulrs():
    __import__(settings.ROOT_URLCONF)
    urls = sys.modules[settings.ROOT_URLCONF]
    urls_pars = []
    for url in urls.urlpatterns:
        if isinstance(url, RegexURLResolver):
            urls_pars += list(map(lambda x: url_pattern(x, url._regex), url.url_patterns))
        else:
            urls_pars += [url_pattern(url, '')]
    return list(filter(lambda x: x[1] is not None, urls_pars))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import datetime
import re
import pytz
from django.conf import settings
from hipster_api.fields.str import String as _String
from hipster_api.fields.number import Integer as _Integer


class DateTime(_String):
    def __init__(self, date_format=None, now=False, timezone=None, **kwargs):
        self.date_format = date_format or '%Y-%m-%d %H:%M:%S'
        self.now = now
        self.timezone_default = timezone or getattr(settings, 'TIME_ZONE', 'Europe/Moscow')
        self.timezone = self.timezone_default
        super(DateTime, self).__init__(**kwargs)

    def __timezone(self):
        timezone = re.search(r'(\(\w+/\w+\))', self.value, re.IGNORECASE)
        if timezone:
            self.timezone = timezone.group(0)[1:-1]
            self.setitem(self.value.replace('(%s)' % self.timezone, ''))
        else:
            self.timezone = self.timezone_default

        return pytz.timezone(self.timezone)

    def to_python(self):
        super(DateTime, self).to_python()
        timezone = self.__timezone()
        value = self.value

        if self.now:
            return self.setitem(datetime.datetime.now())
        try:
            value = datetime.datetime.strptime(value, self.date_format).replace(tzinfo=timezone)
        except ValueError:
            if hasattr(self.default, '__call__'):
                return self.setitem(self.default())
            try:
                value = datetime.datetime.strptime(self.default, self.date_format).replace(tzinfo=timezone)
            except ValueError:
                value = datetime.datetime.now(timezone)
        self.setitem(value)


class Date(DateTime):
    def __init__(self, **kwargs):
        kwargs['date_format'] = '%Y-%m-%d' if 'date_format' not in kwargs else kwargs['date_format']
        super(Date, self).__init__(**kwargs)

    def to_python(self):
        super(Date, self).to_python()
        try:
            self.setitem(self.value.date())
        except AttributeError:
            self.setitem(self.value)


class Boolean(_Integer):
    def to_python(self):
        self.setitem(bool(super(Boolean, self).to_python()))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import codecs
import os
from distutils.core import setup


def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = ['README.md']
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename) for filename in filenames])
    return {package: filepaths}

setup(
    name='django_hipster_api',
    version='1.7.0',
    packages=get_packages('hipster_api'),
    package_data=get_package_data('hipster_api'),
    long_description=read("hipster_api/README.md"),
    install_requires=['pytz>=2015.4', 'djangorestframework>=3.2.2'],
    url='https://github.com/RustoriaRu/hipster_api',
    license='MIT',
    author='vir-mir',
    keywords='django rest framework',
    author_email='virmir49@gmail.com',
    description='wrapper django rest framework',
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)
